#include <bits/stdc++.h>
using ll = long long;
using ld = long double;
using namespace std;

mt19937 mt{ 228 };

using Priority = decltype(mt());
using Key = pair<ll, int>;

struct Node {
    struct Ptr {
        using SplitPair = pair<Ptr, Ptr>;

        Node* value;

        Ptr()
            : value()
        {
        }

        Ptr(Key key)
            : value(new Node(key))
        {
        }
        

        Node* get() const {
            return value;
        }

        bool has() const {
            return value;
        }

        Node* operator -> () const {
            return get();
        }

        explicit operator bool() const {
            return has();
        }

        int size() const {
            return has() ? get()->size : 0;
        }

        static Ptr merge(Ptr lhs, Ptr rhs) {
            if (!lhs) {
                return rhs;
            }

            if (!rhs) {
                return lhs;
            }
            
            Ptr res;

            if (lhs->prior > rhs->prior) {
                res = lhs;
                res->right = merge(lhs->right, rhs);
            } else {
                res = rhs;
                res->left = merge(lhs, rhs->left);
            }

            res->update();
            return res;
        }

        friend Ptr operator + (const Ptr& lhs, const Ptr& rhs) {
            return merge(lhs, rhs);
        }

        SplitPair split(Key key) {
            if (!has()) {
                return {};
            }

            if (get()->key >= key) {
                auto childPair = get()->left.split(key);
                get()->left = childPair.second;
                get()->update();
                return { childPair.first, *this };
            } else {
                auto childPair = get()->right.split(key);
                get()->right = childPair.first;
                get()->update();
                return { *this, childPair.second };
            }
        }

        int greaterThan(ll x) {
            auto childPair = split({ x + 1, 0 });
            int res = childPair.second.size();
            *this = childPair.first + childPair.second;
            return res;
        }

        void erase(Key key) {
            auto childPair = split(key);
            auto rightPair = childPair.second.split({ key.first, key.second + 1 });
            *this = childPair.first + rightPair.second;
        }

        void insert(Key key) {
            auto childPair = split(key);
            *this = childPair.first + Ptr(key) + childPair.second;
        }

    };

    explicit Node(Key key)
        : prior(mt())
        , size(1)
        , key(key)
        , left()
        , right()
    {
    }

    void update() {
        size = left.size() + right.size() + 1;
    }

    Priority prior;
    int size;
    Key key;
    Ptr left;
    Ptr right;
};

using Treap = Node::Ptr;

const int MAXN = 100001;

ll l[MAXN];
ll r[MAXN];
int full[MAXN];

int main() {
#ifdef PAUNSVOKNO
    freopen("input.txt", "r", stdin); freopen("output.txt", "w", stdout);
#endif
    ios_base::sync_with_stdio(false); cout.setf(ios::fixed); cout.precision(20); cout.tie(nullptr); cin.tie(nullptr);
    int n, m, q, k;
    cin >> n >> m >> q >> k;
    for (int i = 0; i < m; ++i) {
        int c;
        cin >> c;
        full[i] = k - c;
    }

    Treap lt, rt;

    for (int i = 0; i < n; ++i) {
        for (int j = 0; j < m; ++j) {
            int x;
            cin >> x;
            l[i] += x;
            r[i] += (x == full[j]) ? k : x;
        }

        lt.insert({ l[i], i });
        rt.insert({ r[i], i });
    }   

    for (int i = 0; i < q; ++i) {
        int t;
        cin >> t;
        if (t == 1) {
            int id;
            cin >> id;
            --id;
            int minPlace = lt.greaterThan(r[id]);
            int maxPlace = rt.greaterThan(l[id]);
            if (l[id] != r[id]) {
                --maxPlace;
            }

            cout << minPlace + 1 << " " << maxPlace + 1 << "\n";
        } else {
            int id, task, score;
            cin >> id >> task >> score;
            --id, --task;
            lt.erase({ l[id], id });
            rt.erase({ r[id], id });
            l[id] += (score - full[task]);
            r[id] -= (k - score);
            lt.insert({ l[id], id });
            rt.insert({ r[id], id });
        }
    }
}