#include <bits/stdc++.h>

using namespace std;

#define X first
#define Y second

typedef long long TreapType;

struct Node;
typedef Node* PNode;

inline int getRand() {
    return (rand() << 15) + rand();
}

struct Node {
    PNode left, right;
    int prior, sz;
    TreapType key;

    Node (TreapType key):
            left(NULL), right(NULL), key(key), prior(getRand()), sz(1) {}
};

inline int getSize(PNode node) {
    return (node ? node->sz : 0);
}

inline void update(PNode node) {
    if (!node)
        return;

    node->sz = 1 + getSize(node->left) + getSize(node->right);
}

PNode merge(PNode treap1, PNode treap2) {
    if (!treap1)
        return treap2;

    if (!treap2)
        return treap1;

    if (treap1->prior > treap2->prior) {
        treap1->right = merge(treap1->right, treap2);
        update(treap1);
        return treap1;
    } else {
        treap2->left = merge(treap1, treap2->left);
        update(treap2);
        return treap2;
    }
}

pair<PNode, PNode> split(PNode treap, TreapType val) {
    if (!treap)
        return {NULL, NULL};

    if (treap->key > val) {
        auto tmp = split(treap->left, val);
        treap->left = tmp.Y;
        update(treap);
        update(tmp.X);
        return {tmp.X, treap};
    } else {
        auto tmp = split(treap->right, val);
        treap->right = tmp.X;
        update(treap);
        update(tmp.Y);
        return {treap, tmp.Y};
    }
}

PNode remove(PNode treap) {
    if (getSize(treap) <= 1)
        return NULL;
        
    if (treap->left)
        treap->left = remove(treap->left);
    else
        treap->right = remove(treap->right);
        
    update(treap);
    
    return treap;
}

void printNode(PNode node) {
    if (!node)
        return;

    printNode(node->left);
    cerr << node->key << ' ';
    printNode(node->right);
}

struct MySet {
    PNode root = NULL;

    inline void insert(TreapType val) {
        auto tmp = split(root, val);
        root = merge(tmp.X, merge(new Node(val), tmp.Y));
    }

    inline void erase(TreapType val) {
        auto tmp1 = split(root, val);
        auto tmp2 = split(tmp1.X, val - 1ll);
        tmp2.Y = remove(tmp2.Y);
        root = merge(merge(tmp2.X, tmp2.Y), tmp1.Y);
    }

    inline int getPlace(TreapType val) {
        auto tmp = split(root, val);
        int res = 1 + getSize(tmp.Y);
        root = merge(tmp.X, tmp.Y);
        return res;
    }
};

int main() {
    ios_base::sync_with_stdio(false);

    int n, m, q, k;
    cin >> n >> m >> q >> k;
    vector<long long> offline(m);
    for (auto& i : offline)
        cin >> i;

    vector< vector<long long> > table(n, vector<long long>(m));
    vector<long long> minSum(n, 0ll), maxSum(n, 0ll);
    for (int i = 0; i < n; i++)
        for (int j = 0; j < m; j++) {
            cin >> table[i][j];
            minSum[i] += table[i][j];
            maxSum[i] += k;
        }

    MySet mx, mn;
    for (int i = 0; i < n; i++) {
        mx.insert(maxSum[i]);
        mn.insert(minSum[i]);
    }

   for (int i = 0; i < q; i++) {
        int t, id;
        cin >> t >> id;
        id--;

         if (t == 1) {
            cout << min(n, mn.getPlace(maxSum[id])) << ' ' << min(n, mx.getPlace(minSum[id]) - (maxSum[id] > minSum[id] ? 1 : 0)) << endl;
        } else {
            int taskId;
            long long ball;
            cin >> taskId >> ball;
            taskId--;

            if (table[id][taskId] + offline[taskId] == k) {
                mx.erase(maxSum[id]);
                mn.erase(minSum[id]);
                minSum[id] += ball - table[id][taskId];
                maxSum[id] -= k - ball;
                mx.insert(maxSum[id]);
                mn.insert(minSum[id]);
            }
        }
    }

    return 0;
}