#include<cstdio>
#include<cstdlib>
#include<cmath>
#include<cassert>
#include<cstring>
#define MAX(a, b) ((a) > (b) ? (a) : (b))
#define MIN(a, b) ((a) < (b) ? (a) : (b))

using namespace std;

enum {
  DIGITS_IN_BASE = 9,
  BASE = 1000 * 1000 * 1000,
  MAXN = 500 * 1000 + 1
};

const int ONE = 1;

char string_number[MAXN + 1];

// a should have enough capacity
void read_number(int *a, int *size, int digits = DIGITS_IN_BASE) {
  scanf("%s", string_number);

  int real_size = 0;
  for (int i = strlen(string_number); i > 0; i -= digits) {
    string_number[i] = 0;
    a[real_size++] = atoi(i >= digits ? string_number + i - digits : string_number);
  }
  *size = real_size;
}

void write_number(const int *a, int size, int digits = DIGITS_IN_BASE) {
  if (size == 0) {
    printf("0\n");
    return;
  }
  printf("%d", a[size - 1]);
  for (int i = size - 2; i >= 0; --i) {    
    printf("%09d", a[i]);
  }
  printf("\n");
}


int upper_pow2(int n) {
  int ans = 1;
  while (ans < n) {
    ans <<= 1;
  }
  return ans;
}

int log2(int n) {
  int ans = 0;
  while (n > 1) {
    n >>= 1;
    ++ans;
  }
  return ans;
}

void set_size(const int *result, int size, int *result_size) {
  int real_size = size;
  while (real_size > 0 && !result[real_size - 1]) {
    --real_size;
  }
  *result_size = real_size;
}

int compare(const int *a, int a_size, const int *b, int b_size) {
  while (a_size > 0 && a_size > b_size) {
    if (a[--a_size]) {
      return 1;
    }
  }

  while (b_size > 0 && b_size > a_size) {
    if (b[--b_size]) {
      return -1;
    }
  }

  // now sizes are equal    
  int a_digit, b_digit;
  for (int i = a_size - 1; i >= 0; --i) {
    a_digit = a[i];
    b_digit = b[i];
    if (a_digit < b_digit) {
      return -1;
    }
    if (a_digit > b_digit) {
      return 1;
    }
  }
  return 0;
}

// a must be >= b
void inplace_subtract(int *a, int a_size, const int *b, int b_size, int *result_size, int base = BASE) {
  int carry = 0, diff;
  for (int i = 0; i < b_size || carry; ++i) {
    diff = a[i] - carry - (i < b_size ? b[i] : 0);
    carry = diff < 0;
    if (carry) {
      diff += base;
    }
    a[i] = diff;
  }
  set_size(a, a_size, result_size);
}

// a must be long enough to contain the result
void inplace_add(int *a, int a_size, const int *b, int b_size, int *result_size) {
  int i, carry = 0;
  for (i = 0; i < MAX(a_size, b_size) || carry; ++i) {
    a[i] = (i < a_size ? a[i] : 0) + carry + (i < b_size ? b[i] : 0);
    carry = a[i] >= BASE;
    if (carry) {
      a[i] -= BASE;
    }
  }
  *result_size = i;
}

void multiply_by_short_ll(const int *a, int a_size, long long b, int *result, int *result_size, int base = BASE) {
  if (a_size == 0) {
    *result_size = 0;
    return;
  }
  long long i, sum, carry = 0;
  for (i = 0; i < a_size || carry; ++i) {
    sum = (i < a_size ? a[i] : 0) * b + carry;
    carry = sum / base;
    result[i] = (int)(sum - carry * base);
  }
  *result_size = i;
}

// returns residual
int inplace_divide_by_short(int *a, int a_size, int b, int *result_size, int base = BASE) {
  if (a_size == 0) {
    *result_size = 0;
    return 0;
  }
  long long sum, carry = 0, b_ll = b;
  for (int i = a_size - 1; i >= 0; --i) {
    sum = a[i] + carry * base;
    a[i] = sum / b;
    carry = sum - a[i] * b_ll;
  }
  set_size(a, a_size, result_size);
  return carry;
}

void convert_base(const int *a, int a_size, int *result, int *result_size, int old_digits, int new_digits) {
  long long p[19];
  p[0] = 1;
  for (int i = 1; i < MAX(old_digits, new_digits) + 1; ++i) {
    p[i] = p[i - 1] * 10;
  }

  long long sum = 0;
  int cur_digits = 0;
  int res_size = 0;
  for (int i = 0; i < a_size; ++i) {
    sum += a[i] * p[cur_digits];
    cur_digits += old_digits;
    while (cur_digits >= new_digits) {
      result[res_size++] = int(sum % p[new_digits]);
      sum /= p[new_digits];
      cur_digits -= new_digits;
    }
  }
  result[res_size++] = sum;
  set_size(result, res_size, result_size);
}

int basecase_sqrt_r[MAXN];
int basecase_sqrt_r1[MAXN];

void basecase_sqrt(
    const int *a, int a_size, 
    int *result, int *result_size, 
    int *residue, int *residue_size, 
    int base = BASE) 
{
  if (a_size == 0) {
    *result_size = 0;
    return;
  }

  int firstDigit;
  if (a_size % 2 == 0) {
    firstDigit = (int)sqrt((long double)a[a_size - 1] * base + a[a_size - 2]);
  } else {
    firstDigit = (int)sqrt((long double)a[a_size - 1]);
  }

  int n = ((a_size + 1) >> 1) << 1;
  int norm = base / (firstDigit + 1);

  memcpy(basecase_sqrt_r, a, a_size * sizeof(int));
  int buf_size = a_size;

  multiply_by_short_ll(basecase_sqrt_r, buf_size, norm, basecase_sqrt_r, &buf_size, base);
  multiply_by_short_ll(basecase_sqrt_r, buf_size, norm, basecase_sqrt_r, &buf_size, base);

  if (buf_size % 2 == 1) {
    basecase_sqrt_r[buf_size++] = 0;
  }
  assert(buf_size == n);

  int res_size = buf_size >> 1;
  int *curr_res = result + res_size;
  int curr_res_size = 0;

  int *curr_r = basecase_sqrt_r + buf_size - 2;
  int curr_r_size = 2;

  firstDigit = (int)sqrt((long double)basecase_sqrt_r[buf_size - 1] * base + basecase_sqrt_r[buf_size - 2]);
  int q = firstDigit;

  int basecase_sqrt_r1_size;
  for (int j = n / 2 - 1; j >= 0; j--) {
    ++q;
    do {
      --q;
      multiply_by_short_ll(curr_res, curr_res_size, 2 * base, basecase_sqrt_r1, &basecase_sqrt_r1_size, base);
      basecase_sqrt_r1[0] = q;
      if (basecase_sqrt_r1_size == 0 && q > 0) {
        basecase_sqrt_r1_size = 1;
      }
      multiply_by_short_ll(basecase_sqrt_r1, basecase_sqrt_r1_size, q,
        basecase_sqrt_r1, &basecase_sqrt_r1_size, base);
    } while (compare(curr_r, curr_r_size, basecase_sqrt_r1, basecase_sqrt_r1_size) < 0);
    inplace_subtract(curr_r, curr_r_size, basecase_sqrt_r1, basecase_sqrt_r1_size, &curr_r_size, base);
    if (j > 0) {
      curr_r -= 2;
      curr_r_size += 2;
    }
    *(--curr_res) = q;
    ++curr_res_size;
    if (j > 0) {
      int d1 = curr_res_size + 2 < curr_r_size ? curr_r[curr_res_size + 2] : 0;
      int d2 = curr_res_size + 1 < curr_r_size ? curr_r[curr_res_size + 1] : 0;
      int d3 = curr_res_size < curr_r_size ? curr_r[curr_res_size] : 0;
      q = ((long long)d1 * base * base + (long long)d2 * base + d3) / (firstDigit * 2);
    }
  }
  assert(curr_res_size == res_size);
  *result_size = res_size;
  long long t = inplace_divide_by_short(result, *result_size, norm, result_size, base);

  // calculating the residual  
  multiply_by_short_ll(result, *result_size, 2 * norm, residue, residue_size, base);
  multiply_by_short_ll(residue, *residue_size, t, residue, residue_size, base);
  long long t2 = t * t;
  int tmp[2] = { t2 % base, t2 / base };
  int tmp_size;
  set_size(tmp, 2, &tmp_size);
  inplace_add(residue, *residue_size, tmp, tmp_size, residue_size);
  inplace_add(residue, *residue_size, curr_r, curr_r_size, residue_size);

  assert(inplace_divide_by_short(residue, *residue_size, norm, residue_size, base) == 0);
  assert(inplace_divide_by_short(residue, *residue_size, norm, residue_size, base) == 0);
}

int n[MAXN], sqrt_s[2 * MAXN], sqrt_r[2 * MAXN], s_plus1_times2[2 * MAXN];
int size, s_size, r_size, s_plus1_times2_size;

int main() {
  read_number(n, &size);
  basecase_sqrt(n, size, sqrt_s, &s_size, sqrt_r, &r_size);
  if (r_size == 0) {
    printf("1\n");
    return 0;
  }
  inplace_add(sqrt_s, s_size, &ONE, 1, &s_size);
  if (compare(sqrt_r, r_size, sqrt_s, s_size) <= 0) {
    write_number(sqrt_r, r_size);
  } else {
    multiply_by_short_ll(sqrt_s, s_size, 2, s_plus1_times2, &s_plus1_times2_size);
    inplace_subtract(s_plus1_times2, s_plus1_times2_size, sqrt_r, r_size, &s_plus1_times2_size);
    write_number(s_plus1_times2, s_plus1_times2_size);
  }  
  return 0;
}